import axios from 'axios';
import qs from 'qs';
import React, { createContext, memo, useContext, useRef, useReducer, useCallback, useEffect, useState } from 'react';

/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

function __awaiter(thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
}

function __generator(thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
}

var makeRequest = function (config, noSerialize) { return __awaiter(void 0, void 0, void 0, function () {
    var response, _a, data, status_1, ex_1, response, data;
    return __generator(this, function (_b) {
        switch (_b.label) {
            case 0:
                _b.trys.push([0, 2, , 3]);
                return [4 /*yield*/, axios(__assign(__assign({}, config), { paramsSerializer: noSerialize ? undefined : qs.stringify }))];
            case 1:
                response = _b.sent();
                _a = response || {}, data = _a.data, status_1 = _a.status;
                return [2 /*return*/, { data: data, status: status_1 }];
            case 2:
                ex_1 = _b.sent();
                response = ex_1.response;
                if ((response === null || response === void 0 ? void 0 : response.status) === 500) {
                    data = (response || {}).data;
                    return [2 /*return*/, {
                            error: {
                                error: { code: (data === null || data === void 0 ? void 0 : data.statusCode) || 500, message: (data === null || data === void 0 ? void 0 : data.code) || 'Server error' },
                                type: 'single',
                            },
                        }];
                }
                if (response === null || response === void 0 ? void 0 : response.data)
                    return [2 /*return*/, response.data];
                return [2 /*return*/, { error: { error: { message: 'Network Error', code: 501 }, type: 'single' } }];
            case 3: return [2 /*return*/];
        }
    });
}); };

var intialState = { headers: undefined, onError: undefined };
var AxiosContext = createContext(intialState);
var useAxiosContext = function () { return useContext(AxiosContext); };
var AxiosProvider = memo(function (props) {
    var children = props.children, onError = props.onError, headers = props.headers;
    return React.createElement(AxiosContext.Provider, { value: { headers: headers, onError: onError } }, children);
});

var initialState = {
    data: null,
};
var axiosReducer = function (state, action) {
    switch (action.type) {
        case 'FETCH':
            return { data: null, loading: true };
        case 'FINISH': {
            var data = action.data, status_1 = action.status;
            return { data: data, loading: false, status: status_1 };
        }
        case 'ERROR': {
            var error = action.error;
            return { data: null, loading: false, error: error };
        }
        case 'CLEAN':
            return initialState;
        default:
            return state;
    }
};

var useAxios = function (url, config, noSerialize) {
    var onInitRef = useRef(false);
    var lastCallRef = useRef({ url: url, method: 'GET' });
    var _a = useAxiosContext(), headers = _a.headers, onErrorCtx = _a.onError;
    var _b = config || {}, onError = _b.onError, onInit = _b.onInit;
    var finalOnError = onError || onErrorCtx;
    var _c = useReducer(function (state, action) { return axiosReducer(state, action); }, initialState), state = _c[0], dispatch = _c[1];
    var call = useCallback(function (config) { return __awaiter(void 0, void 0, void 0, function () {
        var finalUrl, finalConfig, _a, data, status, error;
        return __generator(this, function (_b) {
            switch (_b.label) {
                case 0:
                    finalUrl = (config === null || config === void 0 ? void 0 : config.url) || url;
                    dispatch({ type: 'FETCH', key: url });
                    finalConfig = !config ? lastCallRef.current : __assign({ url: finalUrl }, config);
                    if (headers && finalConfig.headers === undefined) {
                        finalConfig = __assign(__assign({}, finalConfig), { headers: headers });
                    }
                    else if (headers && finalConfig.headers) {
                        finalConfig = __assign(__assign({}, finalConfig), { headers: __assign(__assign({}, headers), finalConfig.headers) });
                    }
                    return [4 /*yield*/, makeRequest(finalConfig, noSerialize)];
                case 1:
                    _a = (_b.sent()) || {}, data = _a.data, status = _a.status, error = _a.error;
                    if (data && status) {
                        lastCallRef.current = finalConfig;
                        dispatch({ type: 'FINISH', key: url, data: data, status: status });
                    }
                    else if (error) {
                        dispatch({ type: 'ERROR', key: url, error: error });
                        if (finalOnError)
                            switch (error.type) {
                                default:
                                case 'single':
                                    finalOnError(error.error);
                                    break;
                                case 'multiple':
                                    error.errors.forEach(function (err) { return finalOnError(err); });
                                    break;
                            }
                    }
                    return [2 /*return*/];
            }
        });
    }); }, [dispatch, headers, url, finalOnError, noSerialize]);
    var clean = useCallback(function () {
        dispatch({ type: 'CLEAN', key: url });
    }, [url, dispatch]);
    useEffect(function () {
        if (onInit && !onInitRef.current) {
            onInitRef.current = true;
            if (!state.data) {
                if (typeof onInit === 'boolean') {
                    if (onInit)
                        call();
                }
                else {
                    call(onInit);
                }
            }
        }
    }, [onInit, call, state.data]);
    return [state.data, state.loading || false, call, clean, state.status, state.error];
};

// eslint-disable-next-line @typescript-eslint/no-explicit-any
var UserContext = createContext([null, function () { }]);
var useUserContext = function () { return useContext(UserContext); };
var UserAxiosProvider = function (props) {
    var firstUpdate = useRef(true);
    var children = props.children, user = props.user, onUser = props.onUser, headers = props.headers, onAxiosError = props.onAxiosError;
    var _a = useState(user), state = _a[0], setState = _a[1];
    useEffect(function () {
        if (firstUpdate.current) {
            firstUpdate.current = false;
            return;
        }
        onUser(state);
    }, [onUser, state]);
    return (React.createElement(UserContext.Provider, { value: [state, setState] },
        React.createElement(AxiosProvider, { onError: onAxiosError, headers: headers }, children)));
};

// import React from 'react'
var useUser = function () {
    var _a = useUserContext(), user = _a[0], setUser = _a[1];
    return { user: user, setUser: setUser };
};

var createUserAxiosCtx = function () {
    return { UserAxiosProvider: UserAxiosProvider, useUser: useUser };
};

export { createUserAxiosCtx, makeRequest, useAxios };
